defmodule NullaWeb.NoteController do
  use NullaWeb, :controller
  alias Nulla.Repo
  alias Nulla.ActivityPub
  alias Nulla.Models.Note
  alias Nulla.Models.InstanceSettings

  def show(conn, %{"username" => username, "id" => id}) do
    accept = List.first(get_req_header(conn, "accept"))
    instance_settings = InstanceSettings.get_instance_settings!()
    domain = instance_settings.domain
    note = Note.get_note!(id) |> Repo.preload([:user, :media_attachments])

    if username != note.user.username do
      conn
      |> put_status(:not_found)
      |> json(%{error: "Note not found"})
      |> halt()
    end

    if accept in ["application/activity+json", "application/ld+json"] do
      conn
      |> put_resp_content_type("application/activity+json")
      |> json(ActivityPub.note(domain, note))
    else
      render(conn, :show, domain: domain, note: note, layout: false)
    end
  end
end
